#!/usr/bin/python
# -*- coding: utf-8 -*-

import os, locale
from PyQt4.QtCore import SIGNAL, Qt
from PyQt4.QtGui import QVBoxLayout, QHBoxLayout, QWidget, QLabel, QSpinBox, QSlider, QStackedWidget, QSizePolicy, QMessageBox
from gui_modules_animation.animation_base_encodageFiltre import Base_EncodageFiltre
from gui_modules_common.mencoder_gui import WidgetMEncoder
### Création des objets: WidgetFFmpeg2theora et WidgetFFmpeg ... l'objet WidgetMEncoder a
### été changé en conséquence #############################################################
from gui_modules_common.ffmpeg2theora_gui import WidgetFFmpeg2theora
from gui_modules_common.ffmpeg_gui import WidgetFFmpeg
from gui_modules_common.EkdWidgets import EkdSaveDialog
from moteur_modules_common.EkdConfig import EkdConfig
from moteur_modules_common.EkdTools import debug
###########################################################################################

# Les widgets des 4 1ères classes apparaissent alternativemement
# selon l'entrée de la boîte de combo sélectionnée (stacked)

class Codecs_SansReglages(QWidget):
	# -------------------------------------------------------------------
	# Widgets associés au codecs:
	# Copie (format original) - AVI-RAW Sans Compression - Codec XVID -
	# Qualite SVCD - Qualite DVD - Codec H264 MPEG 4
	# -------------------------------------------------------------------
	def __init__(self):
		QWidget.__init__(self)

		# label texte
		txt1=_(u"Réglage automatique pour ce choix")
		label=QLabel("<center>%s</center>" %txt1)

		vbox=QVBoxLayout()
		vbox.addWidget(label)
		self.setLayout(vbox)


class Codecs_AvecReglages(QWidget):
	# -------------------------------------------------------------------
	# Widgets associés au codecs :
	# DivX4 - Codec Motion JPEG - Codec MPEG1 - Codec MPEG2 - Codec WMV2
	# -------------------------------------------------------------------
	# Les données sont définies dans l'appel se trouvant dans la classe
	# Animation_Encodage_General car max_range est différent pour codec_vob_ffmpeg
	###
	def __init__(self,choixCodec, min_range, max_range, GoodToBad = True):
		'''
		Codecs_AvecReglages  : Crée un widget avec : 1 slider, 1 bare de progession et un texte explicatif
		          choixCodec : Nom du codec qui sera utilisé lors de l'enregistrement dans EkdConfig
			  min_range  : Valeur minimale du slider et de la barre
			  max_range  : Valeur maximale du slider et de la barre
			  GoodToBad  : Sens du texte explicatif :
			               si True : la valeur de qualité basse correspond à min_range
				       si True : la valeur de qualité basse correspond à max_range
		'''
		QWidget.__init__(self)

		self.choixCodec=choixCodec
		self.idSection = "animation_encodage_general"

		# label titre
		txt1=_(u"Réglage de la compression : ")
		label=QLabel("<center><u>%s</u></center>" %txt1)

		# boite de spin
		self.spinCompression=QSpinBox()		# self car on va récupérer la variable depuis le moteur
		self.spinCompression.setRange(min_range, max_range) # 2 par défaut

		# curseur
		self.curseur = QSlider(Qt.Horizontal)
		self.curseur.setRange(min_range, max_range)

		self.connect(self.spinCompression,SIGNAL("valueChanged(int)"),self.sauverSpin)
		self.connect(self.curseur, SIGNAL("sliderMoved(int)"), self.curseurBouge)

		try:
			debug( "%s" % EkdConfig.get(self.idSection,self.choixCodec))
			self.spinCompression.setValue(int(EkdConfig.get(self.idSection,self.choixCodec)))
		except:
			debug( "Pas de paramètre ou mauvais paramètre de configuration pour:\n\
			DivX4, Codec Motion JPEG, Codec MPEG1, Codec MPEG2 ou Codec WMV2")

		# label d'info des valeurs de la compression video
		txt1,txt2,txt3,txt4=_(u'Entre'),_(u'et'),_(u'Excellent'),_(u'Mauvais')
		if (GoodToBad):
			info_compress=QLabel('%s %d %s %d -> <font color="green">%d : %s</font> -- <font color="red">%d : %s</font>'\
						     %(txt1,min_range,txt2,max_range,min_range,txt3,max_range,txt4), self)
		else:
			info_compress=QLabel('%s %d %s %d -> <font color="green">%d : %s</font> -- <font color="red">%d : %s</font>'\
						     %(txt1,min_range,txt2,max_range,max_range,txt3,min_range,txt4), self)

		# QWidget qui contiendra les widgets ci-dessus ( -> stacked)
		vbox=QVBoxLayout()
		vbox.addWidget(label)
		hbox=QHBoxLayout()
		hbox.addWidget(self.spinCompression)
		hbox.addWidget(self.curseur)
		hbox.addWidget(info_compress)
		hbox.setAlignment(Qt.AlignHCenter)
		vbox.addLayout(hbox)
		self.setLayout(vbox)

	def sauverSpin(self,i):
		"""conserver le spin dans le fichier de configuration et modifie le curseur"""
		debug("%s %d" % (self.choixCodec,i))
		EkdConfig.set(self.idSection,self.choixCodec,i)
		# sauver curseur
		self.curseur.setValue(i)

	def curseurBouge(self, i):
		"""Quand on change la position du curseur la valeur du spin est automatiquement modifiée"""
		self.spinCompression.setValue(i)



class Animation_Encodage_General(Base_EncodageFiltre):
	# -------------------------------------------------------------------
	# Cadre accueillant les widgets de : Animation >> Encodage
	# -------------------------------------------------------------------

	def __init__(self):

		#=== Identifiant de la classe ===#
		self.idSection = "animation_encodage_general"


		# -------------------------------------------------------------------
		# Widgets intégrés à la boîte de groupe "Réglage de sortie de l'encodage".
		# La boîte de combo définie ci-dessus 'sélectionne' ceux qui vont s'afficher
		# -> utilisation d'un stacked
		# -------------------------------------------------------------------

		# === Widgets associés au codecs: Copie (format original) - AVI-RAW Sans
		# Compression - Codec XVID - Qualite SVCD - Qualite DVD - Codec H264 MPEG 4

		# création de stacked pour les QWidget
		self.stacked=QStackedWidget()
		self.stacked.setSizePolicy(QSizePolicy(QSizePolicy.MinimumExpanding, QSizePolicy.Fixed))

		self.itemStacked_sansReglages=Codecs_SansReglages()
		# par défaut
		self.indexStacked_sansReglages=self.stacked.addWidget(self.itemStacked_sansReglages)


		# === Widgets associés au codecs : # DivX4 - Codec Motion JPEG - Codec MPEG1
		# - Codec MPEG2 - Codec WMV2

		self.itemStacked_DIVX4=Codecs_AvecReglages('codecdivx4', min_range=1, max_range=31)
		self.itemStacked_JPEG=Codecs_AvecReglages('codecmotionjpeg', min_range=1, max_range=31)
		self.itemStacked_MPEG1=Codecs_AvecReglages('codecmpeg1', min_range=1, max_range=31)
		self.itemStacked_MPEG2=Codecs_AvecReglages('codecmpeg2', min_range=1, max_range=31)
		self.itemStacked_WMV2=Codecs_AvecReglages('codecwmv2', min_range=1, max_range=31)
		# Ici max_range est à 255 contrairement aux autres
		self.itemStacked_VOB=Codecs_AvecReglages('codec_vob_ffmpeg', min_range=1, max_range=255)
		##################################################################################

		## On utilise le même objet pour le Ogg et le Flash
		self.itemStacked_flash=Codecs_AvecReglages('macromediaflashvideo', min_range=10 , max_range=5000, GoodToBad=False)
		self.itemStacked_oggTheora=Codecs_AvecReglages('codecoggtheora', min_range=0 , max_range=10,  GoodToBad=False)
		self.indexStacked_DIVX4=self.stacked.addWidget(self.itemStacked_DIVX4)
		self.indexStacked_JPEG=self.stacked.addWidget(self.itemStacked_JPEG)
		self.indexStacked_MPEG1=self.stacked.addWidget(self.itemStacked_MPEG1)
		self.indexStacked_MPEG2=self.stacked.addWidget(self.itemStacked_MPEG2)
		self.indexStacked_WMV2=self.stacked.addWidget(self.itemStacked_WMV2)
		self.indexStacked_VOB=self.stacked.addWidget(self.itemStacked_VOB)
		##################################################################################

		# === Widgets associés au codec : Macromedia Flash Video
		self.indexStacked_flash=self.stacked.addWidget(self.itemStacked_flash)

		# === Widgets associés au codec : Ogg Theora
		self.indexStacked_oggTheora=self.stacked.addWidget(self.itemStacked_oggTheora)


		# Réglage rajouté pour l'encodage VOB
		#----------------------------------------------------------------------------------
		# Paramètres de la liste de combo: [(identifiant, nom entrée, index du stacked,
		# 	instance stacked, extension fichier sortie, valeur spin à récupérer?),...]
		#----------------------------------------------------------------------------------
		self.listeCombo =	[\
		('copie',		_(u'Copie (format original)'),		self.indexStacked_sansReglages,\
			self.itemStacked_sansReglages,	'',		0),\
		('avirawsanscompression',_(u'AVI-RAW i420 (sans compression) (.avi)'),	self.indexStacked_sansReglages,\
			self.itemStacked_sansReglages,	'.avi',		0),\
		('codec_dv_ffmpeg',_(u'Codec DV (.dv)'),	self.indexStacked_sansReglages,\
			self.itemStacked_sansReglages,	'.dv',		0),\
		('codec_mov_ffmpeg',_(u'Codec QuickTime MOV (.mov)'),	self.indexStacked_sansReglages,\
			self.itemStacked_sansReglages,	'.mov',		0),\
		('codec_hfyu_ffmpeg',_(u'Codec HFYU: Huffman Lossless YUV (yuv422p) (.avi)'),	self.indexStacked_sansReglages,\
			self.itemStacked_sansReglages,	'.avi',		0),\
		('codecmotionjpeg',	_(u'Codec Motion JPEG (.avi)'),		self.indexStacked_JPEG,\
			self.itemStacked_JPEG,		'.avi',		1),\
		('codecoggtheora',	_(u'Codec OGG THEORA (.ogg)'),		self.indexStacked_oggTheora,\
			self.itemStacked_oggTheora,	'.ogg',		1),\
		('codec_vob_ffmpeg',_(u'Codec VOB (DVD-Video stream MPEG-2) (.vob)'),	self.indexStacked_VOB,\
			self.itemStacked_VOB,	'.vob',		1),\
		('codecmpeg2',		_(u'Codec MPEG 2 (.mpg)'),		self.indexStacked_MPEG2,\
			self.itemStacked_MPEG2,		'.mpg',		1),\
		('codech264mpeg4',	_(u'Codec H264 MPEG 4 (.mp4)'),		self.indexStacked_sansReglages,\
			self.itemStacked_sansReglages,	'.mp4',		0),\
		('codech264mpeg4_ext_h264', _(u'Codec H264 MPEG 4 (.h264)'),	self.indexStacked_sansReglages,\
			self.itemStacked_sansReglages,	'.h264',	0),\
		('codecxvid',		_(u'Codec XVID (.avi)'),		self.indexStacked_sansReglages,\
			self.itemStacked_sansReglages,	'.avi',		0),\
		('codecdivx4',		_(u'Codec DIVX4 (.avi)'),		self.indexStacked_DIVX4,\
			self.itemStacked_DIVX4,		'.avi',		1),\
		('codecmpeg1',		_(u'Codec MPEG 1 (.mpg)'),		self.indexStacked_MPEG1,\
			self.itemStacked_MPEG1,		'.mpg',		1),\
		('macromediaflashvideo',_(u'Macromedia Flash Video (.flv)'),	self.indexStacked_flash,\
			self.itemStacked_flash,		'.flv',		1),\
		('codecwmv2',		_(u'Codec WMV2 (.wmv)'),		self.indexStacked_WMV2,\
			self.itemStacked_WMV2,		'.wmv',		1),\
		('codec_3GP_ffmpeg',_(u'Codec  3GP (3rd Generation Partnership Project) (.3gp)'),	self.indexStacked_sansReglages,\
			self.itemStacked_sansReglages,	'.3gp',		0),\
		('codec_AMV_ffmpeg',		_(u'Codec AMV: pour lecteurs mp4 (.avi)'),	self.indexStacked_sansReglages,\
			self.indexStacked_sansReglages,		'.avi',		0),\
					]

		#---------------------------
		# Dérivation de la classe
		#---------------------------
		super(Animation_Encodage_General, self).__init__(_(u"Transcodage: Général"))



	def afficherAide(self):
		""" Boîte de dialogue de l'aide du cadre Animation > Encodage """
		
		super(Animation_Encodage_General,self).afficherAide(_(u"<p><b>Vous pouvez ici coder ou plutôt transcoder une vidéo d'un format vers un autre. Dans certains cas le transcodage permet de compresser la vidéo, c'est à dire d'en réduire le poids.</b></p><p>Dans l'onglet <b>'Vidéo(s) source'</b> cliquez sur le bouton <b>Ajouter</b>, une boîte de dialogue apparaît, sur la partie gauche sélectionnez le répertoire (au besoin dépliez les sous-répertoires), allez chercher la/les vidéo(s). Si vous voulez sélectionner plusieurs vidéos d'un coup, maintenez la touche <b>CTRL</b> (ou <b>SHIFT</b>) du clavier enfoncée (tout en sélectionnant vos vidéos), cliquez sur <b>Ajouter</b>.</p><p>Vous pouvez dès lors sélectionner une vidéo dans la liste et la visionner (par le bouton juste à la droite de cette liste), vous noterez que vous pouvez visionner la vidéo en quatre tiers, en seize neuvième ou avec les proportions d'origine de la vidéo (w;h). De même si vous le désirez, vous pouvez obtenir des informations complètes sur la vidéo sélectionnée, et ce par le bouton <b>'Infos'</b> (en bas).</p><p>Dans l'onglet <b>'Réglages'</b> sélectionnez le codec, et s'il le faut, faites le <b>'Réglage de la compression'</b>.</p><p>Une fois tout ceci fait, cliquez sur le bouton <b>'Appliquer'</b>, sélectionnez le répertoire de sauvegarde, indiquez votre <b>'Nom de fichier'</b>, cliquez sur le bouton <b>'Enregistrer'</b> et attendez le temps de la conversion. A la fin cliquez sur le bouton <b>'Voir les informations d'encodage'</b> et fermez cette dernière fenêtre après avoir vu les informations en question.</p><p>Dans l'onglet <b>'Visionner vidéo'</b> vous pouvez visionner le résultat (avant la conversion) en sélectionnant <b>'vidéo(s) source(s)'</b>, après la conversion <b>'vidéo convertie'</b> ou bien encore les deux en même temps, en cliquant sur le bouton <b>'Comparateur de vidéos'</b>.</p><p>L'onglet <b>'Infos'</b> vous permet de voir les vidéos chargées (avec leurs chemins exacts) avant et après conversion.</p>"))


	def appliquer(self, nomSortie=None, ouvert=1):
		"""appel du moteur de ekd -> encodage"""

		if not nomSortie:
			index=self.combo.currentIndex()
			chemin = unicode(self.getFile())

			# suffix du codec actif
			suffix=self.listeCombo[index][4]
			if suffix=='':
				suffix=os.path.splitext(chemin)[1]
				
			ext_chargee = os.path.splitext(chemin)[1]
			codec_reglage = self.listeCombo[index][1]
			#######################################################################
			saveDialog = EkdSaveDialog(self, os.path.expanduser(self.repSortieProv()), suffix, _(u"Sauver"))
			cheminFichierEnregistrerVideo = saveDialog.getFile()
			##############################################################################################

		else: # module séquentiel
			chemin=cheminFichierEnregistrerVideo=nomSortie

		if not cheminFichierEnregistrerVideo:
			return

		# quel est l'index du dernier item sélectionné de la boîte de combo?
		i=self.combo.currentIndex()
		# identifiant du codec actif
		idCodec=str(self.combo.itemData(i).toStringList()[0]) # méthode de QVariant

		# pas de spin pour Copie (format original) - AVI-RAW Sans Compression
		# Codec XVID - Qualite SVCD - Qualite DVD - Codec H264 MPEG 4
		spin=None # par défaut

		for j in self.listeCombo: # attribution de la valeur du spin s'il existe
			if j[0]==idCodec and j[5]==1:
				spin=str(j[3].spinCompression.value())

		debug( "%s %s %s %s" % (idCodec, chemin, spin, type(spin)))

		# Appel des classes après séparation des traitements gérés par Mencoder, FFmpeg
		# et FFmpeg2theora
		# appel du moteur -> animation encodage avec un codec particulier
		try:
			##### Ajout de codech264mpeg4_ext_h264 le 12/09/09 #####
			if idCodec in ['copie', 'avirawsanscompression', 'codecdivx4', 'codecmotionjpeg', 'codecmpeg1', 'codecmpeg2', 'codecwmv2', 'codecxvid', 'macromediaflashvideo', 'codech264mpeg4', 'codech264mpeg4_ext_h264']:
				mencoder = WidgetMEncoder(idCodec, chemin, cheminFichierEnregistrerVideo, valeurNum = spin, laisserOuvert=ouvert)
				mencoder.setWindowTitle(_(u"Encodage vidéo"))
				mencoder.exec_()

			elif idCodec == 'codecoggtheora':
				#### Gestion de l'extension .h264 ####
				# Si on charge une vidéo avec extension .h264, ffmpeg2theora ne peut 
				# pas effectuer l'encodage
				if ext_chargee == '.h264':
					messErrExtH264ffmpeg2th=QMessageBox(self)
					messErrExtH264ffmpeg2th.setText(_(u"<p>Il n'est pas possible de donner suite au traitement <b>à partir d'une vidéo avec extension h264 et en ayant choisi: %s.</b></p><p><b>Veuillez choisir un autre codec dans la liste proposée.</b></p>" % codec_reglage))
					messErrExtH264ffmpeg2th.setWindowTitle(_(u"Erreur"))
					messErrExtH264ffmpeg2th.setIcon(QMessageBox.Warning)
					messErrExtH264ffmpeg2th.exec_()
					return
				else:	
					ffmpeg2theora = WidgetFFmpeg2theora(idCodec, chemin, cheminFichierEnregistrerVideo, valeurNum = spin, laisserOuvert=ouvert)
					ffmpeg2theora.setWindowTitle(_(u"Encodage vidéo"))
					ffmpeg2theora.exec_()
				#######################################################################

			else:
				#### Gestion de l'extension .h264 ####
				# Si on charge une vidéo avec extension .h264, FFmpeg ne peut 
				# pas effectuer l'encodage
				if ext_chargee == '.h264':
					if codec_reglage in [u'Codec DV (.dv)', u'Codec QuickTime MOV (.mov)', u'Codec HFYU: Huffman Lossless YUV (yuv422p) (.avi)', u'Codec VOB (DVD-Video stream MPEG-2) (.vob)', u'Codec  3GP (3rd Generation Partnership Project) (.3gp)', u'Codec AMV: pour lecteurs mp4 (.avi)']:
						messErrExtH264ffmpeg=QMessageBox(self)
						messErrExtH264ffmpeg.setText(_(u"<p>Il n'est pas possible de donner suite au traitement <b>à partir d'une vidéo avec extension h264 et en ayant choisi: %s.</b></p><p><b>Veuillez choisir un autre codec dans la liste proposée.</b></p>" % codec_reglage))
						messErrExtH264ffmpeg.setWindowTitle(_(u"Erreur"))
						messErrExtH264ffmpeg.setIcon(QMessageBox.Warning)
						messErrExtH264ffmpeg.exec_()
						return
				else:
					ffmpeg = WidgetFFmpeg(idCodec, chemin, cheminFichierEnregistrerVideo, valeurNum = spin, laisserOuvert=ouvert)
					ffmpeg.setWindowTitle(_(u"Encodage vidéo"))
					ffmpeg.exec_()
				#######################################################################

		#except Exception, e:
		except None:
			messageErreur=QMessageBox(self)
			messageErreur.setText(_(u"Un problème est survenu.")+str(e))
			messageErreur.setWindowTitle(_(u"Erreur"))
			messageErreur.setIcon(QMessageBox.Warning)
			messageErreur.exec_()
			return
		#########################################################################################

		#########################################################################################

		self.lstFichiersSortie =  cheminFichierEnregistrerVideo # pour la boite de dialogue de comparaison
		self.radioConvert.setChecked(True)
		self.radioSource.setEnabled(True)
		self.radioSource.setChecked(False)
		self.radioConvert.setEnabled(True)
		self.boutCompare.setEnabled(True)
		### Information à l'utilisateur
		self.infoLog(None, chemin, None, cheminFichierEnregistrerVideo)

		return cheminFichierEnregistrerVideo # module séquentiel
