#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys, os, string, glob
import Image
from PyQt4.QtCore import *
from PyQt4.QtGui import *

from gui_modules_image.image_base import Base, SpinSliders, SpinSlider

from gui_modules_image.selectWidget import SelectWidget
from gui_modules_lecture.lecture_image import Lecture_VisionImage
from gui_modules_lecture.affichage_image.afficheurImage import VisionneurEvolue

# Gestion de la configuration via EkdConfig
from moteur_modules_common.EkdConfig import EkdConfig
# Nouvelle fenêtre d'aide ######
from gui_modules_common.EkdWidgets import EkdAide
# Nouvelle boite de dialogue pour sauver les fichiers
from gui_modules_common.EkdWidgets import EkdSaveDialog


class Apercu(QWidget):
	"""Aperçu de la découpe/morcellement d'image.
	Concrêtement, dessine la première image d'un lot en traçant dessus des lignes.
	Ces dernières forment des rectangles symbolisant les morceaux qui vont être découpés
	"""
	
	def __init__(self, tailleCanevas=(None, None)):
		QWidget.__init__(self)
		
		# Taille du canevas
		self.tailleCanevas = tailleCanevas
		# Image nulle au départ (rien ne sera donc traçé à ce niveau)
		self.image = QImage()
		# Nombre de cases en horizontal pour la découpe
		self.nbrCaseHoriz = 0
		# Nombre de cases en vertical pour la découpe
		self.nbrCaseVert = 0
	
	def sizeHint(self):
		return QSize(self.tailleCanevas[0], self.tailleCanevas[1])

	def minimumSizeHint(self):
        	return QSize(self.tailleCanevas[0], self.tailleCanevas[1])
	
	def setImage(self, img):
		"Donne l'adresse d'une image à la classe pour ensuite l'afficher"
		image = QImage(img)
		
		largeurImg , hauteurImg = image.size().width(), image.size().height()
		largeurCanevas, hauteurCanevas = self.tailleCanevas
		
		if largeurImg - largeurCanevas > hauteurImg - hauteurCanevas:
			# La largeur est le facteur limitant -> redimension en largeur
			self.image = image.scaledToWidth(largeurCanevas)
		else:
			# La hauteur est le facteur limitant -> redimension en hauteur
			self.image = image.scaledToHeight(hauteurCanevas)
		
		self.repaint()
	
	def setNbrCase(self, nbrCaseHoriz=3, nbrCaseVert=2):
		"Spécifie les paramètres de morcellement de la zone rectangulaire"
		self.nbrCaseHoriz = nbrCaseHoriz
		self.nbrCaseVert = nbrCaseVert
		self.repaint()
	
	def paintEvent(self, event):
		"Trace la première image et les traits qui symboliseront la découpe"
		paint = QPainter()
		paint.begin(self)
		
		# Si aucune image n'a été chargée, on ne trace rien
		if self.image.isNull():
			paint.end()
			return
		
		# Traçage de l'image
		paint.drawImage(0, 0, self.image)
		# On vérifie que l'on découpe véritablement l'image
		if self.nbrCaseHoriz > 1 or self.nbrCaseVert > 1:
			# Largeur de l'image à l'écran
			largeurImgEcran = self.image.size().width()
			# Hauteur de l'image à l'écran
			hauteurImgEcran = self.image.size().height()
			
			# Traçage des lignes verticales
			if self.nbrCaseHoriz > 1:
				# Largeur d'une case
				largeurCase = largeurImgEcran / self.nbrCaseHoriz
				for i in range(self.nbrCaseHoriz):
					if i == 0:
						continue
					paint.drawLine(i * largeurCase, 0, i * largeurCase, hauteurImgEcran)
			
			# Traçage des lignes horizontales
			if self.nbrCaseVert > 1:
				# Hauteur d'une case
				hauteurCase = hauteurImgEcran / self.nbrCaseVert
				for i in range(self.nbrCaseVert):
					if i == 0:
						continue
					paint.drawLine(0, i * hauteurCase, largeurImgEcran, i * hauteurCase)
		
		paint.end()


class Image_Divers_PourLeWeb(QWidget):
	# -----------------------------------
	# Cadre accueillant les widgets de :
	# Image >> Divers >> Pour le web
	# -----------------------------------
	def __init__(self, statusBar, geometry):
        	QWidget.__init__(self)
		
		# ----------------------------
		# Quelques paramètres de base
		# ----------------------------
		
		#=== Création des répertoires temporaires ===#
		# Gestion du repertoire tmp avec EkdConfig 
		self.repTampon = EkdConfig.getTempDir() + os.sep + "tampon" + os.sep + "pour_le_web" + os.sep
		if os.path.isdir(self.repTampon) is False:
        		os.makedirs(self.repTampon)
		if os.path.isdir(self.repTampon+'redim'+os.sep) is False:
        		os.makedirs(self.repTampon+'redim'+os.sep)
			
		# Au cas où le répertoire existait déjà et qu'il n'était pas vide 
		# -> purge (simple précausion)
		for toutRepCompo in glob.glob(self.repTampon+'*.*'):
			if len(toutRepCompo)>0:
				os.remove(toutRepCompo)
			
		for toutRepCompoRedim in glob.glob(self.repTampon+'redim'+os.sep+'*.*'):
			if len(toutRepCompoRedim)>0:
				os.remove(toutRepCompoRedim)
		#############################################################################
		
		#=== Variable contenant les titres du log ===#
		self.infosImgTitre = []
		txt = _(u"Image(s) chargée(s)")
		a='#'*36
		b = a + '\n# ' + txt + '\n' + a + '\n'
		txt=_(u"Image(s) convertie(s)")
		c = a + '\n# ' + txt + '\n' + a + '\n'
		self.infosImgTitre.append(b)
		self.infosImgTitre.append(c)
		
		#=== Drapeaux ===#
		# Une conversion (même partielle) a-t-elle eu lieu après le chargement des images? (1: vrai)
		
		# Est-ce que des images ont été converties et qu'elles n'ont pas encore été montrées?
		# Marche aussi quand la conversion a été arrêté avant la fin de la 1ère image
		self.conversionImg = 0
		
		# Est-ce qu'une prévisualisation a été appelée?
		self.previsualImg = 0
		# Est-ce que des images sources ont été modifiées? (c'est-à-dire ajoutées ou supprimées)
		self.modifImageSource = 0
		
		self.timer = QTimer()
		self.connect(self.timer, SIGNAL('timeout()'), self.sonderTempsActuel)
		
		# Fonctions communes à plusieurs cadres du module Image
		self.base = Base()

		# Gestion de la configuration via EkdConfig

		# Paramètres de configuration
		self.config = EkdConfig
		# Identifiant du cadre
		self.idSection = "image_pour_le_web"
		# Log du terminal
		self.base.printSection(self.idSection)
		# Fonction appelant la fenêtre principale
		self.mainWindowFrameGeometry = geometry

		self.listeImgSource = []
		self.listeImgDestin = []
		
		# Boite d'alignement vertical
		vbox=QVBoxLayout(self)
		
		self.tabwidget=QTabWidget()
		
		#------------------
		# Onglet Réglages
		#------------------
		
		self.framReglage=QFrame()
		vboxReglage=QVBoxLayout(self.framReglage)
		
		# Gestion du nombre d'images à traiter
		self.grid = QGridLayout()
		
		self.labTraitImgNum=QLabel(_(u"Traitement à partir de l'image (numéro)"))
		self.grid.addWidget(self.labTraitImgNum, 0, 0)
		# Label invisible par défaut
		self.labTraitImgNum.hide()
		self.spin1_ImTrait=SpinSlider(1, 100000, 1, '', self)
		self.grid.addWidget(self.spin1_ImTrait, 0, 1)
		# Spin invisible par défaut
		self.spin1_ImTrait.hide()
		self.connect(self.spin1_ImTrait, SIGNAL("valueChanged(int)"), self.changeValNbreImg_1)
		
		self.labNbreChifNIm=QLabel(_(u"Nombre de chiffres après le nom de l'image"))
		self.grid.addWidget(self.labNbreChifNIm, 1, 0)
		# Label invisible par défaut
		self.labNbreChifNIm.hide()
		self.spin2_ImTrait=SpinSlider(3, 18, 6, '', self)
		self.grid.addWidget(self.spin2_ImTrait, 1, 1)
		# Spin invisible par défaut
		self.spin2_ImTrait.hide()
		self.connect(self.spin2_ImTrait, SIGNAL("valueChanged(int)"), self.changeValNbreImg_1)
		
		self.grid.setAlignment(Qt.AlignHCenter)
		vboxReglage.addLayout(self.grid)
		vboxReglage.addStretch()
		
		#=== Stacked ===#
		
		self.stacked = QStackedWidget()
		self.stacked.setSizePolicy(QSizePolicy(QSizePolicy.MinimumExpanding, QSizePolicy.Fixed))
		
		#=== Instanciation des widgets du stacked  ===#
		
		#---- Stacked de Gif
		stacked_gifAnime = QFrame()
		vboxStacked = QVBoxLayout(stacked_gifAnime)
		
		# Widget du stacked avec une seule boite de spin
		self.spinCouleursDelai = SpinSliders(self,
				8, 256, 128, _(u"Nombre de couleurs (Réduction)"), 'nombre_couleurs',
				1, 20000, 10, _(u"Temps de pause entre 2 trames (ms)"), 'delai')
		
		self.spinLargeur = SpinSliders(self, 10, 1024, 200, _(u"Nouvelle largeur (px)"), 'nouvelle_largeur')
		self.spinLargeur.setEnabled(False)
		
		self.checkModifLargeur = QCheckBox("Modifier la largeur de l'image gif")
		self.connect(self.checkModifLargeur, SIGNAL("stateChanged(int)"), self.checkModifLargeurChange)
		self.checkModifLargeur.setChecked(False)
		
		vboxStacked.addWidget(self.spinCouleursDelai, 0)
		vboxStacked.addWidget(QLabel())
		vboxStacked.addWidget(self.checkModifLargeur, 0, Qt.AlignHCenter)
		vboxStacked.addWidget(self.spinLargeur , 0)
		vboxStacked.addStretch()
		
		#---- Stacked de Découpe
		stacked_morcellementImg = QFrame()
		vboxStacked = QVBoxLayout(stacked_morcellementImg)
		self.spinMorcellement = SpinSliders(self,
			1,10,2, _(u"Nombre de morceaux (sens horizontal)"), 'nombre_morceaux_horizontal',
			1,10,2,_(u"Nombre de morceaux (sens vertical)"), 'nombre_morceaux_vertical')
		# Aperçu de la découpe d'image
		self.apercu = Apercu((400, 300))
		self.connect(self.spinMorcellement.spin1, SIGNAL("valueChanged(int)"), self.changeMorcellement)
		self.connect(self.spinMorcellement.spin2, SIGNAL("valueChanged(int)"), self.changeMorcellement)
		self.apercu.setNbrCase(self.spinMorcellement.spin1.value(), self.spinMorcellement.spin2.value())
		vboxStacked.addWidget(self.spinMorcellement)
		hboxMorcellement = QHBoxLayout()
		hboxMorcellement.addStretch()
		hboxMorcellement.addWidget(self.apercu)
		hboxMorcellement.addStretch()
		vboxStacked.addLayout(hboxMorcellement)
		
		# Widgets du stacked avec une seule boite de spin
		indexStacked_gif_anime = self.stacked.addWidget(stacked_gifAnime)
		# Widgets du stacked avec 2 boites de spin
		indexStacked_morcellementImg = self.stacked.addWidget(stacked_morcellementImg)
		
		# boite de combo
		self.comboReglage=QComboBox()
		# Paramètres de la liste de combo: [(nom entrée, identifiant, index du stacked,
		# instance stacked),...]
		self.listeComboReglage=[(_(u'Gif animé'),    'gif_anime',    indexStacked_gif_anime,    self.spinCouleursDelai),\
		(_(u"Morcellement/découpe d'image"),    'morcellement_img',    indexStacked_morcellementImg,    self.spinMorcellement)]
		
		# Insertion des codecs de compression dans la boite de combo
		for i in self.listeComboReglage:
                	self.comboReglage.addItem(i[0], QVariant(i[1]))
		self.connect(self.comboReglage, SIGNAL("currentIndexChanged(int)"), self.changerComboReglage)
		# Affiche l'entrée de la boite de combo inscrite dans un fichier de configuration
		self.base.valeurComboIni(self.comboReglage, self.config, self.idSection, 'methode')
		
		hbox = QHBoxLayout()
		hbox.addWidget(QLabel(_(u'Type')))
		hbox.addWidget(self.comboReglage)
		hbox.setAlignment(Qt.AlignHCenter)
		
		vboxReglage.addLayout(hbox)
		vboxReglage.addWidget(self.stacked)
		vboxReglage.addStretch()
		
		#----------------
		# Onglet de log
		#----------------
		
		self.zoneAffichInfosImg = QTextEdit("")
		if PYQT_VERSION_STR < "4.1.0":
			self.zoneAffichInfosImg.setText = self.zoneAffichInfosImg.setPlainText
		self.zoneAffichInfosImg.setReadOnly(True)
		self.framInfos=QFrame()
		vboxInfIm=QVBoxLayout(self.framInfos)
		vboxInfIm.addWidget(self.zoneAffichInfosImg)
		self.framInfos.setEnabled(False)
		
		# -------------------------------------------------
		# Onglets d'affichage image source et destination
		# -------------------------------------------------
		
		# Là où s'afficheront les images
		self.afficheurImgSource=SelectWidget(geometrie = self.mainWindowFrameGeometry)
		self.afficheurImgDestination=Lecture_VisionImage(statusBar)
		
		vbox.addWidget(self.tabwidget)

		self.indexTabImgSource = self.tabwidget.addTab(self.afficheurImgSource, _(u'Image(s) source'))
		self.indexTabImgReglage=self.tabwidget.addTab(self.framReglage, _(u'Réglages'))
		self.indexTabImgDestin = self.tabwidget.addTab(self.afficheurImgDestination, _(u'Image(s) après traitement'))
		self.indexTabImgInfos=self.tabwidget.addTab(self.framInfos, _(u'Infos'))
		
		# -------------------------------------------
		# widgets du bas : curseur + ligne + boutons
		# -------------------------------------------
		
		# Boutons
		boutAide=QPushButton(_(u" Aide"))
		boutAide.setIcon(QIcon("Icones/icone_aide_128.png"))
		self.connect(boutAide, SIGNAL("clicked()"), self.afficherAide)
		
		self.boutApPremImg = QPushButton(_(u" Voir le résultat"))
		self.boutApPremImg.setIcon(QIcon("Icones/icone_visionner_128.png"))
		self.boutApPremImg.setFocusPolicy(Qt.NoFocus)
		self.boutApPremImg.setEnabled(False)
		self.connect(self.boutApPremImg, SIGNAL("clicked()"), self.visu_1ere_img)
		
		self.boutAppliquer=QPushButton(_(u" Appliquer et sauver"))
		self.boutAppliquer.setIcon(QIcon("Icones/icone_appliquer_128.png"))
		
		# Bouton inactif au départ
		self.boutAppliquer.setEnabled(False)

		self.connect(self.boutAppliquer, SIGNAL("clicked()"), self.appliquer0)

		# Ligne de séparation juste au dessus des boutons
		ligne = QFrame()
		ligne.setFrameShape(QFrame.HLine)
		ligne.setFrameShadow(QFrame.Sunken)
		vbox.addWidget(ligne)
		vbox.addSpacing(-5)	# la ligne doit être plus près des boutons
		
		hbox=QHBoxLayout()
		hbox.addWidget(boutAide)
		hbox.addStretch()
		hbox.addWidget(self.boutApPremImg)
		hbox.addStretch()
		hbox.addWidget(self.boutAppliquer)
		vbox.addLayout(hbox)
		
		self.setLayout(vbox)

		#-----------------------------
		# Barre de progression
		#-----------------------------
		
		self.progress=QProgressDialog(_(u"Conversion en cours..."), _(u"Arrêter"), 0, 100)
		self.progress.setWindowTitle(_(u'EnKoDeur-Mixeur. Fenêtre de progression'))
		self.progress.setMinimumWidth(500)
		self.progress.setMinimumHeight(100)

		self.connect(self.tabwidget, SIGNAL("currentChanged(int)"), self.fctTab)

		#----------------------------------------------------------------------------------------------------
		# Signal de présence d'images dans ler widget de sélection -> modifie le statut des boutons d'action
		#----------------------------------------------------------------------------------------------------
		
		self.connect(self.afficheurImgSource, SIGNAL("pictureChanged(int)"), self.modifImgSource)
		self.connect(self.afficheurImgSource, SIGNAL("pictureSelected()"), self.modifImgSelect)
		
		
	def changeValNbreImg_1(self):
		"""Gestion du nombre d'images à traiter"""
		print "Traitement a partir de l'image (numero):", self.spin1_ImTrait.value()
		print "Nombre de chiffres apres le nom de l'image:", self.spin2_ImTrait.value()


	def modifImgSource(self, i):
		"""On active ou désactive les boutons d'action et on recharge le pseudo-aperçu en fonction du nombre d'images présentes dans le widget de sélection"""
		self.boutAppliquer.setEnabled(i)
		self.boutApPremImg.setEnabled(i)
		self.modifImageSource = 1
		self.apercu.setImage(self.afficheurImgSource.getFile())

	
	def modifImgSelect(self):
		"Une nouvelle image a été sélectionnée. On la donne à l'aperçu"
		self.apercu.setImage(self.afficheurImgSource.getFile())
	
	
	def fctTab(self, i):
		"Affichage d'une ou plusieurs images converties"
		
		# Cela ne concerne que l'onglet de visualisation des images après leur conversion
		if i == self.indexTabImgDestin:
			if self.conversionImg:
				# Affichage si on sauvegarde par le bouton Appliquer et sauver
				print "La conversion vient d'avoir lieu -> affichage des images du lot de destination"
				cheminImages = os.path.dirname(self.listeImgDestin[0])
				liste = []
				for fichier in self.listeImgDestin:
					liste.append(os.path.basename(fichier))
				self.afficheurImgDestination.updateImages(liste, cheminImages)
				# Démarre le *.gif quand ce type d'image a été créée et ne fait rien sinon
				if self.afficheurImgDestination.isGif():
					self.afficheurImgDestination.startPauseGif()
			elif not self.boutAppliquer.isEnabled() or self.modifImageSource:
				# Si le bouton de conversion n'est pas actif, c'est qu'il n'y a plus d'image source
				# -> on n'a plus de raison de maintenir des images dans l'afficheur de résultat
				# Si les images sources ont été modifiées, on purge aussi l'afficheur de résultat
				self.afficheurImgDestination.updateImages([])
			self.conversionImg = 0
			self.modifImageSource = 0
	
	
	def metaFctTab(self, i):
		"""Changement d'onglet (conçu pour sélectionner les onglets "Images Source" après le chargement de nouvelles images sources ou "Images Après Traitement" après la conversion). But: s'assurer que la fonction associée au QTabWidget (affichage d'images, grisage/dégrisage du curseur...) sera bien appliquée même si on est déjà sur le bon onglet"""
		if self.tabwidget.currentIndex()!=i:
			self.tabwidget.setCurrentIndex(i)
		else: self.fctTab(i)
	
	
	def changerComboReglage(self, i):
		"""Récup/affichage ds le terminal de l'index de self.comboReglage"""
		self.stacked.setCurrentIndex(self.listeComboReglage[i][2])
		
		self.config.set(self.idSection, 'methode', self.listeComboReglage[i][1])
		
		print self.comboReglage.currentIndex()
		
		# Si Gif animé est sélectionné, les labels et spins sont invisibles
		if self.comboReglage.currentIndex()==0:
			self.labTraitImgNum.hide()
			self.spin1_ImTrait.hide()
			self.labNbreChifNIm.hide()
			self.spin2_ImTrait.hide()
		# Si Découpe/morcellement est sélectionné, les labels et spins deviennent visibles
		elif self.comboReglage.currentIndex()==1:
			self.labTraitImgNum.show()
			self.spin1_ImTrait.show()
			self.labNbreChifNIm.show()
			self.spin2_ImTrait.show()
		
	
	def changeMorcellement(self, i):
		"Une valeur de morcellement vient de changer. On transmet l'information à l'aperçu"
		self.apercu.setNbrCase(self.spinMorcellement.spin1.value(),
					self.spinMorcellement.spin2.value())
		
		if self.comboReglage.currentIndex()==1:
			self.spin1_ImTrait.show()
	
	
	def checkModifLargeurChange(self, boolean):
		"""La case à cocher change d'état
		-> la boite de spin redimension de la largeur s'active ou se désactive"""
		self.spinLargeur.setEnabled(boolean)
	
	
	def stat_dim_img(self):
		"""Calcul statistique des dimensions des images les plus présentes dans le lot"""
		
		# Récupération de la liste des fichiers chargés
		self.listeImgSource=self.afficheurImgSource.getFiles()
		
		# Ouverture et mise ds une liste des dimensions des images
		listePrepaRedim=[Image.open(aA).size for aA in self.listeImgSource]
		
		# Merci beaucoup à Marc Keller de la liste: python at aful.org de m'avoir
		# aidé pour cette partie (les 4 lignes en dessous)
		dictSeq={}.fromkeys(listePrepaRedim, 0)
		for cle in listePrepaRedim: dictSeq[cle]+=1
		self.lStatDimSeq=sorted(zip(dictSeq.itervalues(), dictSeq.iterkeys()), reverse=1)
		self.dimStatImg=self.lStatDimSeq[0][1]
		
		print "Toutes les dimensions des images (avec le nbre d'images):", self.lStatDimSeq
		print 'Dimension des images la plus presente dans la sequence:', self.dimStatImg
		print "Nombre de tailles d'images différentes dans le lot :", len(self.lStatDimSeq)
		
		if len(self.lStatDimSeq)>1: return 0
		else: return 1
	
	
	def redim_img(self):
		"""Si l'utilisateur charge des images avec des tailles complètement différentes --> les images de la séquence  peuvent être redimensionnées"""
		
		if not self.stat_dim_img():
			reply = QMessageBox.warning(self, 'Message',
			_(u"Vos images ne sont pas toutes de la même taille. Voulez-vous redimensionner les images de sortie à la taille la plus répandue dans la séquence ?. Si vous répondez non, le traitement ne sera pas effectué."), QMessageBox.Yes, QMessageBox.No)
			if reply == QMessageBox.No:
				return
			
			# Les images de tailles différentes à la plus répandue sont redimensionnées
			# dans un répertoire temporaire.
			# Les images redimensionnées voient leur chemin modifié dans la liste des
			# chemins des images sources. Les autres chemins ne changent pas.
			index=0
			for chemImg in self.listeImgSource:
				obImg=Image.open(chemImg)
				if obImg.size!=self.dimStatImg:
					pass
				sRedim=obImg.resize(self.dimStatImg)
				chemSortie = self.repTampon+'redim'+os.sep+os.path.basename(chemImg)
				sRedim.save(chemSortie)
				self.listeImgSource[index] = chemSortie
				index += 1
	
	
	def sonderTempsActuel(self):
		"""x ms après l'apparition de la boite de dialogue, on lance la conversion. But: faire en sorte que la boite de dialogue ait le temps de s'afficher correctement"""
		self.timer.stop()
		self.appliquer()
			
				
	def visu_1ere_img(self):
		"""Visionnement de la conversion en Gif animé et Morcellement/découpe avant application"""
		
		# Si Gif animé est sélectionné
		if self.comboReglage.currentIndex()==0:

			# Récupération de la liste des fichiers chargés car por la visu du gif
			# animé ce sont les 2 1ères images de  la liste qui sont selectionnées
			self.listeImgSource=self.afficheurImgSource.getFiles()

			# Chemin de sauvegarde
			self.cheminCourantSauv = self.repTampon+'gif_anim_visu.gif'
			
			nbreElem=len(self.listeImgSource)

			# Liste de travail pour le redimensionnement
			listeImgTemp=[]
			
			# nbreElem et elem: affectation du nombre d'image et occurence 
			# à utiliser. A ce propos pour la visualisation le nombre d'images
			# utilisées pour la visualisation du gif animé sera de 2 maximum
			# (pour voir le temps de pause entre 2 trames)
			#### Si l'utilisateur ne charge qu'une image ####
			if nbreElem==1:
				nbreElem=1
				elem=0
				ouvrirImag=Image.open(self.listeImgSource[0])
            			ouvrirImag.save(self.repTampon+'redim'+os.sep+'redim_gif_anim_'+string.zfill(1, 3)+'.png')
				listeImgTemp.append(self.repTampon+'redim'+os.sep+'redim_gif_anim_'+string.zfill(1, 3)+'.png')
			#### Si l'utilisateur charge plusieurs images ####
			elif nbreElem>1:
				nbreElem=2
				elem=2
				# ----- Redimensionnement des images --------------------------------------
    				# Ouverture de la 1ere image chargee
				imgOuvComp=Image.open(self.listeImgSource[0])
    				# Dimension de cette image (w--> largeur, h --> hauteur)
    				w, h=imgOuvComp.size
    				# Boucle pour le redimensionnement
    				n=0
				# Tant qu'on a pas atteint le nbre d'images
    				while n < len(self.listeImgSource[0:elem]):
        				for parcRedim in self.listeImgSource[0:elem]:
            					ouvrirImag=Image.open(parcRedim)
            					# Redimensionnement des images
            					sRedimComp=ouvrirImag.resize(imgOuvComp.size)
            					sRedimComp.save(self.repTampon+'redim'+os.sep+'redim_gif_anim_'+string.zfill(n, 3)+'.png')
						listeImgTemp.append(self.repTampon+'redim'+os.sep+'redim_gif_anim_'+string.zfill(n, 3)+'.png')
            					n=n+1	
				
			# Recup de la valeur de Nombre de couleurs (Réduction)
			spinCouleur = str(self.spinCouleursDelai.spin1.value())
			# Nouveau temps de pause entre 2 images
			spinDelai = str(self.spinCouleursDelai.spin2.value())
			if self.spinLargeur.isEnabled():
				# Nouvelle largeur
				spinLargeur = str(self.spinLargeur.spin.value())
			
			listeChemin = ["\"" + i + "\"" for i in listeImgTemp]
			
			commande = 'convert -quiet -delay '+spinDelai+' -loop 0 -colors '+spinCouleur+' '+' '.join(listeChemin)+' '+"\""+self.repTampon+'gif_anim_visu.gif'+"\""
			
			# On ne redimensionne que si la boite de spin associée est active
			if self.spinLargeur.isEnabled():
				commande = commande.replace('convert', 'convert -resize ' + spinLargeur)
					
			os.system(commande)

			# Affichage de l'image temporaire 
			# Ouverture d'une boite de dialogue affichant l'aperçu.
			#
			# Affichage par le bouton Voir le résultat
			visio = VisionneurEvolue(self.cheminCourantSauv)
			visio.redimenFenetre(self.mainWindowFrameGeometry, 1., 0.7)
			visio.startPauseGif()
			visio.exec_()
		
			return 0
			
		# Si Découpe/morcellement est sélectionné	
		elif self.comboReglage.currentIndex()==1:
			
			# Récupération du fichier sélectionné par l'utilisateur (si pas de fichier
			# sélectionné par l'utilisateur, la 1ère image de la liste est prise)
			file = self.afficheurImgSource.getFile()
			if not file: return
			self.listeImgSource = [file]
			
			# Récupération de l'identifiant du codec
			i = self.comboReglage.currentIndex()
			
			# Recup de la valeur de Nombre de morceaux (sens horizontal)
			spin_1 = self.listeComboReglage[i][3].spin1.value()
			# Recup de la valeur de Nombre de morceaux (sens vertical)
			spin_2 = self.listeComboReglage[i][3].spin2.value()
			
			# Ouverture de la 1ère image du lot pour extension
			im=Image.open(self.listeImgSource[0])
			# Chemin, extension
			f, ext=os.path.splitext(self.listeImgSource[0])
			
			# Résolution de l'image
			resLarg, resHaut=im.size
			
			# Comprehension-list pour calculer les coordonnées de découpe dans le sens horizontal
			l_1=[(totoX*(resLarg/spin_1), totoY*(resHaut/spin_2)) for totoY in range(spin_2) for totoX in range(spin_1)]
			# Comprehension-list pour calculer les coordonnées de découpe dans le sens vertical
			l_2=[(totoX*(resLarg/spin_1)+(resLarg/spin_1), totoY*(resHaut/spin_2)+(resHaut/spin_2)) for totoY in range(spin_2) for totoX in range(spin_1)]
			# Regroupement sens horizontal et vertical
			inter=zip(l_1, l_2)
			# Regroupement final des coordonnées horizontales et verticales pour la découpe.
			# Affiche par exemple: [(0, 0, 105, 144), (105, 0, 210, 144), (210, 0, 315, 144), ...]
			lF=[(x1[0], x1[1], y1[0], y1[1]) for x1, y1 in inter]
			
			# Marge (en pixels) pour l'affichage
			marge=6
			# Calcul des coordonnées pour affichage dans l'interface
			lCoAff=[(((tX+1)*marge)+(tX*(resLarg/spin_1)), ((tY+1)*marge)+(tY*(resHaut/spin_2))) for tY in range(spin_2) for tX in range(spin_1)]
			# Largeur de l'image de fond pour affichage ds l'interface
			largFondAffichage=resLarg+((spin_1+1)*marge)
			# Hauteur de l'image de fond pour affichage ds l'interface
			hautFondAffichage=resHaut+((spin_2+1)*marge)
			# Creation de l'image de fond pour acceuillir les morceaux de l'image (affichage
			# dans l'interface)
			imgFond=Image.new('RGB', (largFondAffichage, hautFondAffichage), (119, 119, 121))
			
			# Regroupement des coordonnées horizontales et verticales pour la découpe et des
			# coordonnées d'affichage des images découpées dans l'interface.
			lFinal=zip(lF, lCoAff)
			
			# Parcours pour la découpe de chaque image
			for a1 in enumerate(lFinal):
				
				# Index du morceau
				nb=a1[0]
				# Coordonnées de la découpe sous la forme (a, b, c, d)
				cDecoup=a1[1][0]
				# Coordonnées d'affichage ds l'interface (a, b)
				cAffInterfac=a1[1][1]
					
				# Ouverture de la 1ère image du lot
				im=Image.open(self.listeImgSource[0])
				
				# Voir doc: http://www.pythonware.com/library/pil/handbook/introduction.htm
				# Découpe ...
				c=im.crop(cDecoup)
				# Pour les commentaires de cette partie en particulier, voir dans la fonction appliquer
				c.save(self.repTampon+'_'+string.zfill(1, 3)+'_'+string.zfill(nb+1, 3)+'_'+str(spin_1)+'x'+str(spin_2)+ext)
				
				# Ré-ouverture ...
				im2=Image.open(self.repTampon+'_'+string.zfill(1, 3)+'_'+string.zfill(nb+1, 3)+'_'+str(spin_1)+'x'+str(spin_2)+ext)
				
				# Copie des images découpées dans l'image de fond
				imgFond.paste(im2, cAffInterfac)
				# Sauvegarde dans le rep. tampon pour affichage dans l'interface
				imgFond.save(self.repTampon+'dec_im_'+string.zfill(1, 3)+'_'+string.zfill(nb+1, 3)+'_'+str(spin_1)+'x'+str(spin_2)+'.jpg')

			# Chemin de sauvegarde
			self.cheminCourantSauv=self.repTampon+'dec_im_'+string.zfill(1, 3)+'_'+string.zfill(nb+1, 3)+'_'+str(spin_1)+'x'+str(spin_2)+'.jpg'

			# Affichage de l'image temporaire 
			# Ouverture d'une boite de dialogue affichant l'aperçu.
			#
			# Affichage par le bouton Voir le résultat
			visio = VisionneurEvolue(self.cheminCourantSauv)
			visio.redimenFenetre(self.mainWindowFrameGeometry, 1., 0.7)
			visio.exec_()
		
			return 0


	def appliquer0(self):
		'''Préparation de la conversion'''
		
		'''
		C'est quoi ce truc, c'est un bug ??
		
		On se met ds la situation où on charge (par exemple) 3 images et qu'on sélectionne:
		Image > Pour le web > Morcellement/découpe d'image, on clique sur Appliquer et sauver,
		la barre de  progression fonctionne bien, les images découpées sont bien affichées et
		enregistrées sur le disque. Maintenant on ds l'onglet Image(s) source et on supprime
		une image (bouton Retirer), on clique sur Appliquer et sauver ... EKD se ferme brutalement
		et on a ce message ds la console:
		
		ASSERT failure in QVector<T>::at: "index out of range", file ../../include/QtCore/../../src/corelib/tools/qvector.h, 
		line 316
		Abandon
		
		Cela ne se produit que si j'active self.listeImgDestin=[], si désactivé aucun souci mais ...
		
		Le problème est qu'il faut vider la liste self.listeImgDestin car si elle n'est pas vidée
		les images issues de la découpe (et les gifs animés) s'accumulent irrémédiablement ... et 
		ce ds le cas de plusieurs traitements successifs. Donc obligé de désactiver 
		self.listeImgDestin=[] pour l'instant.
		'''
		
		# Si l'utilisateur charge des images de taille différente, fait le traitement 
		# ...,  recharge de nouvelles images, il faut que les répertoires de redimen-
		# sionnement soient vidés
		
		listePresRedim_1=glob.glob(self.repTampon+'redim_1'+os.sep+'*.*')
		if len(listePresRedim_1)>0:
			for parcR_1 in listePresRedim_1: os.remove(parcR_1)

		# Récupération de la liste des fichiers chargés
		self.listeChemin=self.afficheurImgSource.getFiles()
		
		# Redimensionner les images de tailles différentes?
		self.redim_img()
		
		# Récupération du chemin
		rep = self.base.getRepSource(self.config)

		# Utilisation de la nouvelle boîte de dialogue de sauvegarde
		suffix=""
		self.chemDossierSauv = EkdSaveDialog(self, os.path.expanduser(rep), suffix, _(u"Sauver"), multiple=True)
		self.chemDossierSauv = self.chemDossierSauv.getFile()
		
		if not self.chemDossierSauv: return
		
		# Récupération de l'identifiant du codec
		self.i = self.comboReglage.currentIndex()
		
		self.progress.reset()
		self.progress.show()
		self.progress.setValue(0)
		QApplication.processEvents()
		
		# Lancement de la conversion dans 250 ms (seule solution trouvée pour éviter le grisage au début)
		self.timer.start(250)
	
	
	def appliquer(self):
		"""Conversion des images"""
		
		# Liste pour affichage des images chargées (ds le tabwidget)
		listeAff_1=[]
		# Liste pour affichage des images sauvegardées (ds le tabwidget)
		listeAff_2=[]

		# Si Gif animé est sélectionné
		if self.comboReglage.currentIndex()==0:
			
			# La liste pour l'affichage des images ds l'interface est
			# vidée pour que les images affichées ne s'amoncellent pas
			# si plusieurs rendus à la suite
			if len(self.listeImgDestin)>0: del self.listeImgDestin[:]
			
			nbreElem=len(self.listeImgSource)
			
			# Recup de la valeur de Nombre de couleurs (Réduction)
			spinCouleur = str(self.spinCouleursDelai.spin1.value())
			# Nouveau temps de pause entre 2 images
			spinDelai = str(self.spinCouleursDelai.spin2.value())
			if self.spinLargeur.isEnabled():
				# Nouvelle largeur
				spinLargeur = str(self.spinLargeur.spin.value())
			
			listeChemin = ["\"" + i + "\"" for i in self.listeImgSource]
			
			commande = 'convert -quiet -delay '+spinDelai+' -loop 0 -colors '+spinCouleur+' '+' '.join(listeChemin)+' '+"\""+self.chemDossierSauv+'.gif'+"\""
			
			# On ne redimensionne que si la boite de spin associée est active
			if self.spinLargeur.isEnabled():
				commande = commande.replace('convert', 'convert -resize ' + spinLargeur)
			
			# Mise en place des conditions pour Linux et Windows
			
			# Uniquement pour Linux et MacOSX
			if os.name in ['posix', 'mac']:
				
				process = QProcess(self)
				process.start(commande)
				if not process.waitForStarted(3000):
					QMessageBox.warning(None, _(u"Erreur"), _(u"Bogue au lancement de la commande"))
				process.waitForFinished(-1)
				
			# Uniquement pour windows
			elif os.name == 'nt':
				
				# Attention (sous Windows !!!) si on appelle une commande exterieure
				# (c'est a dire une commande non Python [par PIL ou Numpy par
				# exemple]) par QProcess, la commande en question ne sera pas
				# executee ... et aucun warning ou message d'erreur ne sera fourni
				# !!!. Il faut obligatoirement passer par os.system (!)
				os.system(commande)
			
			for parc in range(nbreElem):
			
				# Remplissage de la liste pour les statistiques de rendu
				listeAff_1.append(self.listeImgSource[parc])
				
				# --------------------------------------------
				# Affichage de la progression (avec
				# QProgressDialog) ds une fenêtre séparée .
				val_pourc=((parc+1)*100)/nbreElem

				# Bouton Cancel pour arrêter la progression donc le process
				if (self.progress.wasCanceled()): break

				self.progress.setValue(val_pourc)
			
				QApplication.processEvents()
				# --------------------------------------------

			# Ajout des images par la variable self.cheminCourantSauv dans la liste 
			# self.listeImgDestin. Cette liste sert à récupérer les images pour l'affichage 
			# des images ds l'inteface.
			self.listeImgDestin.append(self.chemDossierSauv+'.gif')
			
			# Affichage des images après traitement
			#
			# Changement d'onglet et fonctions associées
			self.conversionImg = 1
			self.metaFctTab(self.indexTabImgDestin)
			
			# Affichage des infos sur l'image -------------------------	
			# On implémente les chemins des fichiers dans une variable
			# pour préparer l'affichage des infos
			texte1=_(u" Image(s) chargée(s)")
			texte2=_(u" Gif animé final")
			a='#'*36
			
			self.infosImgProv_1=a+'\n#'+texte1+'\n'+a
			self.infosImgProv_2=a+'\n#'+texte2+'\n'+a
		
			# Images chargées
			for parcStatRendu_1 in listeAff_1:
				self.infosImgProv_1=self.infosImgProv_1+'\n'+parcStatRendu_1
			
			# Gif animé sauvegardé
			self.infosImgProv_2=self.infosImgProv_2+'\n'+self.chemDossierSauv+'.gif'
		
			# affichage des infos dans l'onglet
			self.zoneAffichInfosImg.setText(self.infosImgProv_1+'\n\n'+self.infosImgProv_2+'\n\n')
			self.framInfos.setEnabled(True)
			
			# remise à 0 de la variable provisoire de log
			self.infosImgProv_1=''
			self.infosImgProv_2=''
			# ---------------------------------------------------------

		# Si Morcellement/découpe d'image est sélectionné	
		elif self.comboReglage.currentIndex()==1:

			# La liste pour l'affichage des images ds l'interface est
			# vidée pour que les images affichées ne s'amoncellent pas
			# si plusieurs rendus à la suite
			if len(self.listeImgDestin)>0: del self.listeImgDestin[:]
			
			# Recup de la valeur de Nombre de morceaux (sens horizontal)
			spin_1 = self.listeComboReglage[self.i][3].spin1.value()
			# Recup de la valeur de Nombre de morceaux (sens vertical)
			spin_2 = self.listeComboReglage[self.i][3].spin2.value()
			
			# Ouverture de la 1ère image du lot pour extension
			im=Image.open(self.listeImgSource[0])
			# Chemin, extension
			f, ext=os.path.splitext(self.listeImgSource[0])
			
			# Résolution de l'image
			resLarg, resHaut=im.size
			
			# Comprehension-list pour calculer les coordonnées de découpe dans le sens horizontal
			l_1=[(totoX*(resLarg/spin_1), totoY*(resHaut/spin_2)) for totoY in range(spin_2) for totoX in range(spin_1)]
			# Comprehension-list pour calculer les coordonnées de découpe dans le sens vertical
			l_2=[(totoX*(resLarg/spin_1)+(resLarg/spin_1), totoY*(resHaut/spin_2)+(resHaut/spin_2)) for totoY in range(spin_2) for totoX in range(spin_1)]
			# Regroupement sens horizontal et vertical
			inter=zip(l_1, l_2)
			# Regroupement final des coordonnées horizontales et verticales pour la découpe.
			# Affiche par exemple: [(0, 0, 105, 144), (105, 0, 210, 144), (210, 0, 315, 144), ...]
			lF=[(x1[0], x1[1], y1[0], y1[1]) for x1, y1 in inter]
			
			# Marge (en pixels) pour l'affichage
			marge=6
			# Calcul des coordonnées pour affichage dans l'interface
			lCoAff=[(((tX+1)*marge)+(tX*(resLarg/spin_1)), ((tY+1)*marge)+(tY*(resHaut/spin_2))) for tY in range(spin_2) for tX in range(spin_1)]
			# Largeur de l'image de fond pour affichage ds l'interface
			largFondAffichage=resLarg+((spin_1+1)*marge)
			# Hauteur de l'image de fond pour affichage ds l'interface
			hautFondAffichage=resHaut+((spin_2+1)*marge)
			# Creation de l'image de fond pour acceuillir les morceaux de l'image (affichage
			# dans l'interface)
			imgFond=Image.new('RGB', (largFondAffichage, hautFondAffichage), (119, 119, 121))
			
			# Regroupement des coordonnées horizontales et verticales pour la découpe et des
			# coordonnées d'affichage des images découpées dans l'interface.
			lFinal=zip(lF, lCoAff)
			
			nbreElem_2=len(self.listeImgSource)
			
			# Parcours principal des images chargées par l'utilisateur
			for parcImg in range(nbreElem_2):
				# Parcours pour la découpe de chaque image
				for a1 in enumerate(lFinal):
				
					# Index du morceau
					nb=a1[0]
					# Coordonnées de la découpe sous la forme (a, b, c, d)
					cDecoup=a1[1][0]
					# Coordonnées d'affichage ds l'interface (a, b)
					cAffInterfac=a1[1][1]
				
					print 'Index', nb, 'Coord decoupe', cDecoup, 'Coord affichage', cAffInterfac
					
					# Ouverture de chaque image
					im=Image.open(self.listeImgSource[parcImg])
					
					# ---------------------------------------------------------
					# Sauvegarde de la découpe sur le disque dur
					# ---------------------------------------------------------
					
					cheminSauv=self.chemDossierSauv+'_'+string.zfill(parcImg+self.spin1_ImTrait.value(), self.spin2_ImTrait.value())+'_'+string.zfill(nb+1, 3)+'_'+str(spin_1)+'x'+str(spin_2)+ext
				
					# Voir doc: http://www.pythonware.com/library/pil/handbook/introduction.htm
					# Découpe ...
					c=im.crop(cDecoup)
					# Attention (et ce pour chaque fichier qui sera sauvegardé --> utilisateur
					# ou rep. temporaire) le 1er chiffre correspondra à l'image en cours, le 
					# second chiffre correspondra au morceau découpé en cours, la dernière
					# partie correspond au nombre de morceaux sur chaque ligne ... le signe
					# multiplié ... le nombre de morceaux sur chaque colonne. Exemple:
					# /.../.../.../nom_image_003_006_3x2.jpg
					# --> il s'agit de la 3ème image et du 6ème morceau découpé ... et nous
					# avons 3 images sur chaque ligne et 2 sur chaque colonne.
					c.save(cheminSauv)
					
					# ---------------------------------------------------------
					# Sauvegarde pour affichage des images dans l'interface
					# ---------------------------------------------------------
					
					# Sauvegarde des images et construction des plaques reconstituées d'images 
					# pour l'affichage dans l'interface
					c.save(self.repTampon+'aff_img_decoup_'+string.zfill(parcImg+self.spin1_ImTrait.value(), self.spin2_ImTrait.value())+'_'+string.zfill(nb+1, 3)+'_'+str(spin_1)+'x'+str(spin_2)+ext)
					# Ré-ouverture ...
					im2=Image.open(self.repTampon+'aff_img_decoup_'+string.zfill(parcImg+self.spin1_ImTrait.value(), self.spin2_ImTrait.value())+'_'+string.zfill(nb+1, 3)+'_'+str(spin_1)+'x'+str(spin_2)+ext)
					# Copie des images découpées dans l'image de fond (la plaque)
					imgFond.paste(im2, cAffInterfac)
					# Sauvegarde dans le rep. tampon pour affichage dans l'interface
					imgFond.save(self.repTampon+'aff_img_decoup_'+string.zfill(parcImg+self.spin1_ImTrait.value(), self.spin2_ImTrait.value())+'_'+string.zfill(nb+1, 3)+'_'+str(spin_1)+'x'+str(spin_2)+ext)
					
					# ---------------------------------------------------------
					# Infos
					# ---------------------------------------------------------
					
					# Remplissage de la liste pour les statistiques de rendu
					listeAff_1.append(self.listeImgSource[parcImg])
				
					# Remplissage de la liste des pages sauvegardées (Info)
					listeAff_2.append(cheminSauv)
				
					# La page Image résultat devient visible à la fin des conversions
					#self.tabwidget.setCurrentIndex(self.indexTabImgDestin)
					
					# --------------------------------------------
					# Affichage de la progression (avec
					# QProgressDialog) ds une fenêtre séparée .
					val_pourc=((parcImg+1)*100)/nbreElem_2
					#val_pourc=((parcImg+1)*100)/self.nbrImg

					# Bouton Cancel pour arrêter la progression donc le process
					if (self.progress.wasCanceled()): break

					self.progress.setValue(val_pourc)
			
					QApplication.processEvents()
					# --------------------------------------------

				# Ajout des images pour l'affichage dans la liste self.listeImgDestin.
				# Les images en question sont les morceaux découpés qui sont ensuite 
				# reconstitués sur la plaque (image de fond) et réaffichés dans l'interface
				self.listeImgDestin.append(self.repTampon+'aff_img_decoup_'+string.zfill(parcImg+self.spin1_ImTrait.value(), self.spin2_ImTrait.value())+'_'+string.zfill(nb+1, 3)+'_'+str(spin_1)+'x'+str(spin_2)+ext)

				# Affichage des images après traitement
				#
				# Changement d'onglet et fonctions associées
				self.conversionImg = 1
				self.metaFctTab(self.indexTabImgDestin)

			# Récupération du chemin + nom des fichiers
			# sauvegardés en ayant éliminé les doublons
			listeAff_1=list(set(listeAff_1))
			listeAff_1.sort()
			
			# Affichage des infos sur l'image -------------------------	
			# On implémente les chemins des fichiers dans une variable
			# pour préparer l'affichage des infos
			texte1=_(u" Image(s) chargée(s)")
			texte2=_(u" Image(s) et morceaux découpés")
			a='#'*36
			
			self.infosImgProv_1=a+'\n#'+texte1+'\n'+a
			self.infosImgProv_2=a+'\n#'+texte2+'\n'+a
		
			# Images chargées
			for parcStatRendu_1 in listeAff_1:
				self.infosImgProv_1=self.infosImgProv_1+'\n'+parcStatRendu_1
			
			# Pages sauvegardées
			for parcStatRendu_2 in listeAff_2:
				self.infosImgProv_2=self.infosImgProv_2+'\n'+parcStatRendu_2
		
			# affichage des infos dans l'onglet
			self.zoneAffichInfosImg.setText(self.infosImgProv_1+'\n\n'+self.infosImgProv_2+'\n\n')
			self.framInfos.setEnabled(True)
			
			# remise à 0 de la variable provisoire de log
			self.infosImgProv_1=''
			self.infosImgProv_2=''
			# ---------------------------------------------------------

	
	def afficherAide(self):
		"""Boîte de dialogue de l'aide"""
		# Nouvelle fenêtre d'aide
		messageAide=EkdAide(parent=self)
		messageAide.setText(tr(u"<p><b>Vous pouvez ici faire quelques traitements concernant la publication d'images sur internet, spécifiquement: gif animé et morcellement/découpe d'image.</b></p><p>Dans l'onglet <b>'Image(s) source'</b> cliquez sur le bouton <b>Ajouter</b>, une boîte de dialogue apparaît, sur la partie gauche sélectionnez le répertoire (au besoin dépliez les sous-répertoires), allez chercher vos image(s). Si vous voulez sélectionner plusieurs images d'un coup, maintenez la touche <b>CTRL</b> (ou <b>SHIFT</b>) du clavier enfoncée (tout en sélectionnant vos images), cliquez sur <b>Ajouter</b>.</p><p>Dans l'onglet <b>'Réglages'</b> sélectionnez votre <b>'Type'</b> (<b>Gif animé</b> ou <b>Morcellement/découpe d'image</b>), faites le/les réglage(s) correspondant au <b>'Type'</b> choisi. Si vous avez choisi <b>Morcellement/découpe d'image</b> vous voyez apparaître <b>'Traitement à partir de l'image (numéro)'</b> et <b>'Nombre de chiffres après le nom de l'image'</b> changez en les valeurs si vous le désirez <b><font color='red'>(la plupart du temps celles par défaut suffisent)</font></b>.</p><p>Cliquez sur le bouton <b>'Voir le résultat'</b>, vous voyez à ce moment là le résultat de vos réglages s'afficher dans une nouvelle fenêtre. A ce sujet et concernant la transformation en <b>'Gif animé'</b>, le résultat de la conversion se fait uniquement sur les deux premières images du lot ... et cela pour (entre autre) voir en images ce que donne le réglage du <b>'Temps de pause entre 2 trames (ms)'</b>.</p><p>Une fois tout ceci fait, cliquez sur le bouton <b>'Appliquer et sauver'</b>, sélectionnez le répertoire de sauvegarde, indiquez votre <b>'Nom de fichier'</b>, cliquez sur le bouton <b>'Enregistrer'</b>.</p><p>Si vous faites un clic droit de la souris (sur l'image) dans l'onglet <b>'Images après traitement'</b>, vous accédez à des paramètres vous permettant différents affichages de la dite image. De même dans cet onglet vous pouvez lancer la visualisation des images par le bouton <b>Lancer le diaporama</b> (le bouton violet avec une flèche blanche vers la droite) et ce dans le cas de <b>Morcellement/découpe d'image</b>, pour ce qui est de lancer l'animation pour un gif animé, faites un clic droit sur l'image dans la fenêtre de rendu et cliquez soit sur le petit bouton <b>Mettre en pause/démarrer l'animation</b>, soit sur <b>Arrêter l'animation</b>.</p><p>L'onglet <b>'Infos'</b> vous permet de voir les image(s) chargée(s) et les image(s) convertie(s).</p>"))
		messageAide.show()


	def save(self) :
		self.afficheurImgSource.saveFileLocation(self.idSection)
		EkdConfig.set(self.idSection, u'spin1_ImTrait', unicode(self.spin1_ImTrait.value()))
		EkdConfig.set(self.idSection, u'spin2_ImTrait', unicode(self.spin2_ImTrait.value()))
		EkdConfig.set(self.idSection, u'spinCouleursDelai', unicode(self.spinCouleursDelai.value()))
		EkdConfig.set(self.idSection, u'spinLargeur1', unicode(self.spinLargeur.spin.value()))
		EkdConfig.set(self.idSection, u'spinLargeur2', unicode(self.spinLargeur.spin2.value()))
		EkdConfig.set(self.idSection, u'spinMorcellement1', unicode(self.spinMorcellement.spin.value()))
		EkdConfig.set(self.idSection, u'spinMorcellement2', unicode(self.spinMorcellement.spin2.value()))
		EkdConfig.set(self.idSection, u'checkModifLargeur', unicode(int(self.checkModifLargeur.isChecked())))
		EkdConfig.set(self.idSection, u'format_sortie', unicode(self.comboReglage.currentIndex()))


	def load(self) :
		self.afficheurImgSource.loadFileLocation(self.idSection)
		self.spin1_ImTrait.setValue(int(EkdConfig.get(self.idSection, 'spin1_ImTrait')))
		self.spin2_ImTrait.setValue(int(EkdConfig.get(self.idSection, 'spin2_ImTrait')))
		self.spinCouleursDelai.setValue(int(EkdConfig.get(self.idSection, 'spinCouleursDelai')))
		self.spinLargeur.spin.setValue(int(EkdConfig.get(self.idSection, 'spinLargeur1')))	
		self.spinLargeur.spin2.setValue(int(EkdConfig.get(self.idSection, 'spinLargeur2')))	
		self.spinMorcellement.spin.setValue(int(EkdConfig.get(self.idSection, 'spinMorcellement1')))	
		self.spinMorcellement.spin2.setValue(int(EkdConfig.get(self.idSection, 'spinMorcellement2')))	
		self.checkModifLargeur.setChecked(int(EkdConfig.get(self.idSection, 'checkModifLargeur')))
		self.comboReglage.setCurrentIndex(int(EkdConfig.get(self.idSection, 'format_sortie')))
