#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
# On importe le strict nécessaire
from PyQt4.QtCore import SIGNAL, SLOT, Qt
from PyQt4.QtGui import QWidget, QSpinBox, QSlider, QHBoxLayout, QGridLayout, QLabel
# Utilisation de EkdConfig
from moteur_modules_common.EkdConfig import EkdConfig


class Base(object):
	#-------------------------------------
	# Fonctions communes au module Image
	#-------------------------------------

	def getRepSource(self, config):
		"Renvoie le répertoire source qui sera utilisée dans la boite de dialogue de fichiers"
		try:
			repEntree = config.get('general','image_input_path').decode("UTF8")
		except Exception, e:
			repEntree = '~'
		if not os.path.exists(repEntree):
			repEntree = '~'

		return os.path.expanduser(repEntree)


	def setRepSource(self, config, cheminEntree):
		"Sauvegarde le répertoire source qui sera utilisée dans la boite de dialogue de fichiers"
		if isinstance(cheminEntree, list):
			if len(cheminEntree)==0: return
			config.set('general','image_input_path',os.path.dirname(cheminEntree[0]).encode("UTF8"))

		elif isinstance(cheminEntree, unicode):
			if not cheminEntree: return
			config.set('general','image_input_path',os.path.dirname(cheminEntree).encode("UTF8"))


	def valeurComboIni(self, comboBox, config, idSection, optionConfig):
		"Affiche au lancement du logiciel l'entrée de la boite de combo inscrite dans un fichier de configuration"
		try:
			valeur = config.get(idSection, optionConfig)
		except:
			print "Pas de valeur dans la config pour : ", idSection, optionConfig
		else:
			codec_trouve = False
			for i in range(comboBox.count()):
				if comboBox.itemData(i).toString() == valeur:
					indice = i
					codec_trouve = True
			if not codec_trouve: indice = 0
			comboBox.setCurrentIndex(indice)


	def printSection(self, idSection):
		"""Affichage dans le terminal de la section initialisée"""
		print '\n'+'-'*30+'\n| '+idSection+'\n'+'-'*30+'\n'


class SpinSlider(QWidget):
	"Boite de spin + curseur avec sauvegarde de la dernière valeur sélectionnée"
	def __init__(self, debut, fin, defaut, optionConfig=None, parent=None):
		QWidget.__init__(self)

		self.spin = QSpinBox()
		self.slider = QSlider(Qt.Horizontal)
		self.spin.setRange(debut, fin)
		self.slider.setRange(debut, fin)

		valeur = None
		if optionConfig:
			self.config = parent.config
			self.idSection = parent.idSection
			self.optionConfig = optionConfig
			try :
				valeur = self.config.get(self.idSection, self.optionConfig)
			except:
				valeur = defaut
				self.config.set(self.idSection, self.optionConfig, valeur)
		erreur = False
		if valeur:
			try: valeur = int(valeur)
			except ValueError: erreur = True
		if not erreur and debut <= valeur <= fin:
			self.spin.setValue(valeur)
			self.slider.setValue(valeur)
		else:
			self.spin.setValue(defaut)
			self.slider.setValue(defaut)
		if optionConfig: self.connect(self.spin, SIGNAL("valueChanged(int)"), self.sauveConfig)
		self.connect(self.spin, SIGNAL("valueChanged(int)"), self.slider,
						SLOT("setValue(int)"))
		self.connect(self.slider, SIGNAL("valueChanged(int)"), self.spin,
						SLOT("setValue(int)"))
		self.connect(self.spin, SIGNAL("valueChanged(int)"), self.changed)
		hbox = QHBoxLayout(self)
		hbox.addWidget(self.spin)
		hbox.addWidget(self.slider)
		hbox.setMargin(0) # pour un meilleur alignement dans un grid

	def changed(self, i):
		self.emit(SIGNAL("valueChanged(int)"), i)

	def sauveConfig(self, i):
		self.config.set(self.idSection, self.optionConfig, str(i))

	def value(self):
		return self.spin.value()

	def setValue(self, i):
		self.spin.setValue(i)
		self.slider.setValue(i)


class SpinSliders(QWidget):
	"Boites de spin + curseur avec sauvegarde de la dernière valeur sélectionnée"
	def __init__(self, parent,
				debut1, fin1, defaut1, txt1, optionConfig1,
				debut2=None, fin2=None, defaut2=None, txt2=None, optionConfig2=None,
				debut3=None, fin3=None, defaut3=None, txt3=None, optionConfig3=None,
				):
		QWidget.__init__(self)
		grid = QGridLayout(self)
		self.spin = SpinSlider(debut1, fin1, defaut1, optionConfig1, parent)
		self.spin1= self.spin
		grid.addWidget(self.spin,0,0)
		if txt1: grid.addWidget(QLabel(txt1),0,1)
		if debut2:
			self.spin2 = SpinSlider(debut2, fin2, defaut2, optionConfig2, parent)
			grid.addWidget(self.spin2,1,0)
			if txt2: grid.addWidget(QLabel(txt2),1,1)
		if debut3:
			self.spin3 = SpinSlider(debut3, fin3, defaut3, optionConfig3, parent)
			grid.addWidget(self.spin3,2,0)
			if txt3: grid.addWidget(QLabel(txt3),2,1)
		grid.setAlignment(Qt.AlignCenter)
		grid.setMargin(0)
