# -*- coding: utf-8 -*-

import os, string, subprocess
from PyQt4.QtCore import SIGNAL, SLOT, QThread

from moteur_modules_common.EkdProcess import EkdProcess
from moteur_modules_common.EkdTools import debug

############################################################################################
# Merci à l'auteur de: http://snippets.prendreuncafe.com/snippets/tagged/pyqt/order_by/date
# pour son code ... et son aide
############################################################################################

class FfmpegAvchd(QThread):

	def __init__(self, cheminVideoEntre, enregistrerSortie, codec_sortie, reso_largeur_sortie, reso_hauteur_sortie, nbreImgSec_sortie, qualite_sortie, cheminFFmpeg=None):

		QThread.__init__(self)
		
		self.cancel = False
		
		#=== Paramètres généraux ===#
		
		# Uniquement pour Linux et MacOSX
		if os.name in ['posix', 'mac']: self.cheminFFmpeg = "ffmpeg"
			
		# Uniquement pour windows
		elif os.name == 'nt': self.cheminFFmpeg = "ffmpeg.exe"

		# chemin(s) des vidéos chargées
		self.cheminVideoEntre = cheminVideoEntre
		# Chemin + nom de fichier en sortie
		self.enregistrerSortie = enregistrerSortie
		# Sélection du codec en sortie (par l'utilisateur)
		self.codec_sortie = codec_sortie
		# Sélection de la largeur (vidéo) en sortie (par l'utilisateur)
		self.reso_largeur_sortie = reso_largeur_sortie
		# Sélection de la hauteur (vidéo) en sortie (par l'utilisateur)
		self.reso_hauteur_sortie = reso_hauteur_sortie
		# Sélection du nbr d'img/sec en sortie (par l'utilisateur)
		self.nbreImgSec_sortie = nbreImgSec_sortie
		# Sélection de la qualité (de la vidéo) en sortie (par l'utilisateur)
		self.qualite_sortie = qualite_sortie
	
	
	def run(self):
		
		for nb_1, fichier_1 in enumerate(self.cheminVideoEntre):
			
			if self.cancel: break
			
			if self.codec_sortie == "MOV (.mov)":
				fps = ' -r '+str(self.nbreImgSec_sortie)
				extSortie = '.mov'
				vcodec = ' -vcodec mpeg4'
				acodec = ' -acodec copy'
				
			elif self.codec_sortie == "MPEG2 (.mpg)":
				fps = ' -r 25'
				extSortie = '.mpg'
				vcodec = ' -vcodec mpeg2video'
				acodec = ' -acodec mp2 -ac 1 -ar 22050 -ab 64k'
				
			elif self.codec_sortie == "MPEG1 (.mpg)":
				fps = ' -r 25'
				extSortie = '.mpg'
				vcodec = ' -vcodec mpeg1video'
				acodec = ' -acodec mp2 -ac 1 -ar 22050 -ab 64k'
				
			elif self.codec_sortie == "VOB (.vob)":
				fps = ' -r '+str(self.nbreImgSec_sortie)
				extSortie = '.vob'
				vcodec = ' '
				acodec = ' -acodec copy'
				
			elif self.codec_sortie == "MPEG4 (.mp4)":
				fps = ' -r '+str(self.nbreImgSec_sortie)
				extSortie = '.mp4'
				vcodec = ' -vcodec mpeg4'
				acodec = ' -acodec copy'
				
			elif self.codec_sortie == "WMV2 (.wmv)":
				fps = ' -r '+str(self.nbreImgSec_sortie)
				extSortie = '.wmv'
				vcodec = ' -vcodec wmv2'
				acodec = ' -acodec copy'
					
			elif self.codec_sortie == "HFYU (yuv422p) (.avi)":
				fps = ' -r '+str(self.nbreImgSec_sortie)
				extSortie = '.avi'
				vcodec = ' -vcodec huffyuv -pix_fmt yuv422p'
				acodec = ' -acodec copy'
				
			elif self.codec_sortie == "MSMPEG 4 version 2 (.avi)":
				fps = ' -r '+str(self.nbreImgSec_sortie)
				extSortie = '.avi'
				vcodec = ' -vcodec msmpeg4v2'
				acodec = ' -acodec copy'
				
			elif self.codec_sortie == "Motion JPEG (.avi)":
				fps = ' -r '+str(self.nbreImgSec_sortie)
				extSortie = '.avi'
				vcodec = ' -vcodec mjpeg'
				acodec = ' -acodec copy'
				
			elif self.codec_sortie == "FFV1 (FFmpeg) (.avi)":
				fps = ' -r '+str(self.nbreImgSec_sortie)
				extSortie = '.avi'
				vcodec = ' -vcodec ffv1'
				acodec = ' -acodec copy'
				
			commande = self.cheminFFmpeg+' -i '+"\""+fichier_1+"\""+' -s '+self.reso_largeur_sortie+'x'+self.reso_hauteur_sortie+fps+vcodec+vcodec+acodec+' -qscale '+self.qualite_sortie+' -y '+"\""+self.enregistrerSortie+'_'+string.zfill(str(nb_1+1), 5)+extSortie+"\""
			outputfile = self.enregistrerSortie+'_'+string.zfill(str(nb_1+1), 5)+extSortie

			try:
				# Travail avec subprocess
				#sp = subprocess.Popen(commande, shell=True, stdin=None, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, preexec_fn=os.setsid)
				sp = EkdProcess(commande, outputerr=subprocess.STDOUT)
				# Important permet le traitement (!)
				self.tampon = sp.stdout.readlines()
			except Exception:
				debug(u"Erreur dans le lancement de %s" % commande)
			
			# Ca marche mieux avec pourcent calculé comme ça
			pourcent=((nb_1+1)*100)/len(self.cheminVideoEntre)

			# Emetteur pour la récup des valeurs du pourcentage
			self.emit(SIGNAL("increment(int)"), pourcent)

			# Emetteur pour (essayer) d'afficher chaque fichier chargé
			# (pour info) au bon moment dans la barre de progression
			self.emit(SIGNAL("travail_avec_fichier"), fichier_1, outputfile)
			
			debug ("%d "%(nb_1))
	
		self.emit(SIGNAL("fin process"))
	
	def cancel(self):
		"""SLOT to cancel treatment"""
		self.cancel = True


